/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1999
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Seth Spitzer <sspitzer@netscape.com>
 *   Robert Ginda <rginda@netscape.com>
 *   Justin Arthur <justinarthur@ieee.org>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/*
 * This file contains the following MozPhone related components:
 * 1. Command line handler service, for responding to the -phone command line
 *    option.
 * 2. Protocol handler for supplying a channel to the browser when an tel://
 *
 * Borrowed from ChatZilla. Thanks !
 */

/* components defined in this file
	MozPhone UUID is 4de2cc0c-a9e4-4853-871c-7cda72582e13 */
const CLINE_SERVICE_CONTRACTID =
    "@mozilla.org/commandlinehandler/general-startup;1?type=phone";
const CLINE_SERVICE_CID =
    Components.ID("{da2a5256-8e9c-4b05-a9cc-d80e249fd493}");
const TELCONTENT_LISTENER_CONTRACTID =
    "@mozilla.org/uriloader/tel-external-content-listener;1";
const TELCONTENT_LISTENER_CID =
    Components.ID("{ca83f67b-d850-48d1-afa2-057b6d3969f0}");
const TELPROT_HANDLER_CONTRACTID =
    "@mozilla.org/network/protocol;1?name=tel";
const TELPROT_HANDLER_CID =
    Components.ID("{bb225d9f-4c7d-40a8-aa0a-c758c875b5e2}");

const TEL_MIMETYPE = "application/x-tel";

const MEDIATOR_CONTRACTID =
    "@mozilla.org/appshell/window-mediator;1";
const STANDARDURL_CONTRACTID = 
    "@mozilla.org/network/standard-url;1";
const IOSERVICE_CONTRACTID = 
    "@mozilla.org/network/io-service;1";
const ASS_CONTRACTID =
    "@mozilla.org/appshell/appShellService;1";
const RDFS_CONTRACTID =
    "@mozilla.org/rdf/rdf-service;1";

/* interfaces used in this file */
const nsIWindowMediator  = Components.interfaces.nsIWindowMediator;
const nsICmdLineHandler  = Components.interfaces.nsICmdLineHandler;
const nsICategoryManager = Components.interfaces.nsICategoryManager;
const nsIContentHandler  = Components.interfaces.nsIContentHandler;
const nsIURIContentListener = Components.interfaces.nsIURIContentListener;
const nsIURILoader       = Components.interfaces.nsIURILoader;
const nsIProtocolHandler = Components.interfaces.nsIProtocolHandler;
const nsIURI             = Components.interfaces.nsIURI;
const nsIStandardURL     = Components.interfaces.nsIStandardURL;
const nsIChannel         = Components.interfaces.nsIChannel;
const nsIRequest         = Components.interfaces.nsIRequest;
const nsIIOService       = Components.interfaces.nsIIOService;
const nsIAppShellService = Components.interfaces.nsIAppShellService;
const nsISupports        = Components.interfaces.nsISupports;
const nsISupportsWeakReference = Components.interfaces.nsISupportsWeakReference;
const nsIRDFService      = Components.interfaces.nsIRDFService;
const nsICommandLineHandler = Components.interfaces.nsICommandLineHandler;
const nsICommandLine     = Components.interfaces.nsICommandLine;

/**************************************************************/
function find_mozphone(num) {
	var ifaces = Components.interfaces;
	var mediator = Components.classes["@mozilla.org/appshell/window-mediator;1"].
		getService(ifaces.nsIWindowMediator);
	var winEnum = mediator.getXULWindowEnumerator('');
	while (winEnum.hasMoreElements()){
		var win = winEnum.getNext();
		if (!(win instanceof ifaces.nsIXULWindow))
			continue;
		var iface = ifaces.nsIDocShellTreeItem;
		var docShellEnum = win.docShell.getDocShellEnumerator(
			iface.typeAll, iface.ENUMERATE_FORWARDS);
		while(docShellEnum.hasMoreElements()) {
			var docShell = docShellEnum.getNext();
			if(docShell instanceof ifaces.nsIDocShell) {
				var d = docShell.contentViewer.DOMDocument;
				if (d.location=='chrome://mozphone/content/mozphone.xul' &&
						d.getElementById) {
					dump('MozPhone found ! Calling '+typeof(d)+' ('+num+')\n' );
					d.getElementById('call_adr').value = num ;
					d.getElementById('bcall').click();
					return true;
				}
			}
		}
	}
	dump('MozPhone NOT found !\n' );
	return false;
}

/**
 * Opens a chrome window.
 * @param aChromeURISpec a string specifying the URI of the window to open.
 * @param aArgument an argument to pass to the window (may be null)
 */
function openWindow(aChromeURISpec, aArgument)
{
  var ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"].
    getService(Components.interfaces.nsIWindowWatcher);
  ww.openWindow(null, aChromeURISpec, "_blank",
                "chrome,menubar,toolbar,status,resizable,dialog=no",
                aArgument);
}
 
/**
/* Command Line handler service */
function CLineService()
{ dump('mp_CLineService\n'); }

CLineService.prototype.commandLineArgument = "-phone";
CLineService.prototype.prefNameForStartup = "general.startup.phone";
CLineService.prototype.chromeUrlForTask = "chrome://mozphone/content/";
CLineService.prototype.helpText = "Start with an IAX softphone";
CLineService.prototype.handlesArgs = true;
CLineService.prototype.defaultArgs = "";
CLineService.prototype.openWindowWithArgs = true;
CLineService.prototype.helpInfo = "  -phone [<number>]  Start with an IAX2 softphone client.\n";

/* nsISupports */
CLineService.prototype.QueryInterface =
function handler_QueryInterface(iid)
{
dump('mp_CLineService.prototype.QueryInterface\n');
    if (iid.equals(nsISupports))
        return this;

    if (nsICmdLineHandler && iid.equals(nsICmdLineHandler))
        return this;

    if (nsICommandLineHandler && iid.equals(nsICommandLineHandler))
        return this;

dump('mp_CLineService.prototype.QueryInterface ==> ERROR\n');
    throw Components.results.NS_ERROR_NO_INTERFACE;
}

/* nsICommandLineHandler */
CLineService.prototype.handle =
function handler_handle(cmdLine)
{
dump('mp_CLineService.prototype.handle\n');
    var args;
    try
    {
        var uristr = cmdLine.handleFlagWithParam("phone", false);
        if (uristr)
        {
            args = new Object();
            args.url = uristr;
        }
    }
    catch (e)
    {
    }

		// Load Mozphone preferences
      var url = "@mozilla.org/moz/jssubscript-loader;1";
		var srv = "mozIJSSubScriptLoader";
		Components.classes[url].getService(Components.interfaces[srv]).loadSubScript('chrome://mozphone/content/jslib.js');
		Components.classes[url].getService(Components.interfaces[srv]).loadSubScript('chrome://mozphone/content/preferences.js');
		var prefs = new Preferences();
		prefs.load();
		if( prefs.auto_start == 'true'
				|| args || cmdLine.handleFlag("phone", false)) {
			dump('mp_comp: auto_start=true or phone cmdline, starting MozPhone !\n');
/*			mozphone_loaded = 'mozphone_loading';
			if (prefs.gui =='sidebar')
					browserWindow.toggleSidebar('mp_sidebar');
			else */
			mozphone_loading	= 'window';
			openWindow('chrome://mozphone/content/mozphone.xul', null);
/*		}

    if (args || cmdLine.handleFlag("phone", false))
    {
        var assClass = Components.classes[ASS_CONTRACTID];
        var ass = assClass.getService(nsIAppShellService);
        var hWin = ass.hiddenDOMWindow;
        hWin.openDialog("chrome://mozphone/content/", "_blank",
                        "chrome,menubar,toolbar,status,resizable,dialog=no",
                        args);

/* find_mozphone();
var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
var browserWindow = wm.getMostRecentWindow("navigator:browser");
//browserWindow.toggleSidebar('mp_sidebar');
*/
			if( args || cmdLine.handleFlag("phone", false)) cmdLine.preventDefault = true;
    }
}

/* factory for command line handler service (CLineService) */
var CLineFactory = new Object();

CLineFactory.createInstance =
function (outer, iid)
{
dump('clinefactory_createInstance\n');
    if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;

/*    if (!iid.equals(nsICmdLineHandler) && !iid.equals(nsISupports))
        throw Components.results.NS_ERROR_INVALID_ARG; */

    return new CLineService().QueryInterface(iid);
}

/* content listener */
function TELContentListener(isSecure)
{
    this.isSecure = isSecure;
}

TELContentListener.prototype.QueryInterface =
function telcl_QueryInterface(iid)
{
dump('telcl_QueryInterface\n');
    if (!iid.equals(nsIURIContentListener) && 
        !iid.equals(nsISupportsWeakReference) &&
        !iid.equals(nsISupports))
    {
        throw Components.results.NS_ERROR_NO_INTERFACE;
    }

    return this;
}

TELContentListener.prototype.loadCookie = null;
TELContentListener.prototype.parentContentListener = null;

TELContentListener.prototype.onStartURIOpen =
function telcl_onStartURIOpen(uri)
{
}

TELContentListener.prototype.doContent =
function telcl_doContent(contentType, preferred, request, contentHandler, count)
{
dump( 'telcl_doContent\n' );

    var e;
	 var channel = request.QueryInterface(nsIChannel);

    var wmClass = Components.classes[MEDIATOR_CONTRACTID];
    var windowManager = wmClass.getService(nsIWindowMediator);

    var assClass = Components.classes[ASS_CONTRACTID];
    var ass = assClass.getService(nsIAppShellService);
    hiddenWin = ass.hiddenDOMWindow;

    // Ok, not starting currently, so check if we've got existing windows.
    var w = windowManager.getMostRecentWindow("tel:mozphone");
 
    // Claiming that a MozPhone window is loading.
    if ("MozPhoneStarting" in hiddenWin) {
dump("mp-service: Mozphone claiming to be starting.\n");
        if (w && ("client" in w) && ("initialized" in w.client) && 
            w.client.initialized)
        {
dump("mp-service: It lied. It's finished starting.\n");
            // It's actually loaded ok.
            delete hiddenWin.MozPhoneStarting;
        }
    }
    
    if ("MozPhoneStarting" in hiddenWin) {
        count = count || 0;
        
        if ((new Date() - hiddenWin.MozPhoneStarting) > 10000) {
            dump("mp-service: Continuing to be unable to talk to existing window!\n");
        } else {
            //dump("cz-service: **** Try: " + count + ", delay: " + (new Date() - hiddenWin.ChatZillaStarting) + "\n");
            
            // We have a MozPhone window, but we're still loading.
            hiddenWin.setTimeout(function wrapper(t, count) {
                    t.doContent(contentType, preferred, request, contentHandler, count + 1);
                }, 250, this, count);
            return true;
        }
    }

    // MozPhone lives !
	var num = channel.URI.host + channel.URI.path;
    if (w) {
dump("mp-service: Existing, fully loaded window. Using.\n");
        // Mozphone is working and initialized ok. Use it.
        w.focus();
        w.gui.call_adr.value = num;
        w.call();
        return true;
    }
    dump("mp-service: No windows, starting new one.\n");
    // Ok, no available window, loading or otherwise, so start MozPhone.
    var args = new Object();
    args.url = channel.URI.host;
 
    hiddenWin.MozPhoneStarting = new Date();
//    hiddenWin.openDialog("chrome://mozphone/content/mozphone.xul",
//       "_blank", "chrome,menubar,toolbar,status,resizable,dialog=no", args);
	if (!find_mozphone(num)) {
		var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].
					getService(Components.interfaces.nsIWindowMediator);
		var browserWindow = wm.getMostRecentWindow("navigator:browser");
		browserWindow.toggleSidebar('mp_sidebar');
		find_mozphone(num);
	}
	return true;
}

TELContentListener.prototype.isPreferred =
function telcl_isPreferred(contentType, desiredContentType)
{
dump('telcl_isPreferred\n');
    return (contentType == TEL_MIMETYPE );
}

TELContentListener.prototype.canHandleContent =
function telcl_canHandleContent(contentType, isContentPreferred, desiredContentType)
{
dump('telcl_canHandleContent\n');
    return (contentType == TEL_MIMETYPE );
}

/* protocol handler factory object (TELContentListener) */
var TELContentListenerFactory = new Object();

TELContentListenerFactory.createInstance =
function telclf_createInstance(outer, iid)
{
dump('telclf_createInstance\n');
    if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;
    
    if (!iid.equals(nsIURIContentListener) && 
        !iid.equals(nsISupportsWeakReference) &&
        !iid.equals(nsISupports))
    {
        throw Components.results.NS_ERROR_INVALID_ARG;
    }
    
    return new TELContentListener(false);
}


/* tel protocol handler component */
function TELProtocolHandler(isSecure)
{
dump('TELProtocolHandler isSecure=' + isSecure + '\n');
    this.isSecure = isSecure;
}

TELProtocolHandler.prototype.protocolFlags = 
                   nsIProtocolHandler.URI_NORELATIVE;

TELProtocolHandler.prototype.allowPort =
function telph_allowPort(port, scheme)
{
dump('telph_allowPort\n');
    return false;
}

TELProtocolHandler.prototype.newURI =
function telph_newURI(spec, charset, baseURI)
{
dump('telph_newURI ' + baseURI +'\n');

    var cls = Components.classes[STANDARDURL_CONTRACTID];
    var url = cls.createInstance(nsIStandardURL);
    url.init(nsIStandardURL.URLTYPE_STANDARD, 9999, spec, charset, baseURI);

    return url.QueryInterface(nsIURI);
}

TELProtocolHandler.prototype.newChannel =
function telph_newChannel(URI)
{
dump('telph_newChannel\n');
    ios = Components.classes[IOSERVICE_CONTRACTID].getService(nsIIOService);
    if (!ios.allowPort(URI.port, URI.scheme))
        throw Components.results.NS_ERROR_FAILURE;
    
    var bogusChan = new BogusChannel(URI, this.isSecure);
    bogusChan.contentType = TEL_MIMETYPE;
    return bogusChan;
}

/* protocol handler factory object (TELProtocolHandler) */
var TELProtocolHandlerFactory = new Object();

TELProtocolHandlerFactory.createInstance =
function telphf_createInstance(outer, iid)
{
dump('telphf_createInstance\n');
    if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;
    
    if (!iid.equals(nsIProtocolHandler) && !iid.equals(nsISupports))
        throw Components.results.NS_ERROR_INVALID_ARG;
    
    var protHandler = new TELProtocolHandler(false);
    protHandler.scheme = "tel";
    protHandler.defaultPort = -1;
    return protHandler;
}

/* bogus channel used by the TELProtocolHandler */
function BogusChannel(URI, isSecure)
{
dump('BogusChannel\n');
    this.URI = URI;
    this.originalURI = URI;
    this.isSecure = isSecure;
}

BogusChannel.prototype.QueryInterface =
function bc_QueryInterface(iid)
{
dump('bc_QueryInterface\n');
if (!iid.equals(nsIChannel) && !iid.equals(nsIRequest) &&
    !iid.equals(nsISupports))
    dump('Components.results.NS_ERROR_NO_INTERFACE\n');
    
    if (!iid.equals(nsIChannel) && !iid.equals(nsIRequest) &&
        !iid.equals(nsISupports))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    
    return this;
}

/* nsIChannel */
BogusChannel.prototype.loadAttributes = null;
BogusChannel.prototype.contentLength = 0;
BogusChannel.prototype.owner = null;
BogusChannel.prototype.loadGroup = null;
BogusChannel.prototype.notificationCallbacks = null;
BogusChannel.prototype.securityInfo = null;

BogusChannel.prototype.open =
BogusChannel.prototype.asyncOpen =
function bc_asyncOpen(observer, ctxt)
{
dump('bc_asyncOpen\n');
    observer.onStartRequest(this, ctxt);
}

BogusChannel.prototype.asyncRead =
function bc_asyncRead(listener, ctxt)
{
dump('bc_asyncRead\n');
    return listener.onStartRequest(this, ctxt);
}

/* nsIRequest */
BogusChannel.prototype.isPending =
function bc_isPending()
{
dump('bc_isPending\n');
    return true;
}

BogusChannel.prototype.status = Components.results.NS_OK;

BogusChannel.prototype.cancel =
function bc_cancel(status)
{
dump('bc_cancel\n');
    this.status = status;
}

BogusChannel.prototype.suspend =
BogusChannel.prototype.resume =
function bc_suspres()
{
dump('bc_suspres\n');
    throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
}

var MozphoneModule = new Object();

/*
MozphoneModule.registerSelf =
function mp_mod_registerSelf(compMgr, fileSpec, location, type)
{
dump('mp_mod_registerSelf\n');
}
*/

const COMPONENT_CONTRACTID =
    "@mozilla.org/mozphone_component;1";
const COMPONENT_CID =
    Components.ID("{6f82910b-fc08-4fa3-85a5-6668e3c2c32b}");

// Component definition
var mozphone_loaded	= '';
var mozphone_loading	= '';
function MozphoneComponent() {}

MozphoneComponent.prototype = {
	start: function (how) {
		if (mozphone_loading != '') // already started ?
			return false;
		dump( 'starting as ' + how + '\n' );
		mozphone_loading	= how;
	},
	stop: function () {
		mozphone_loaded = '';
		mozphone_loading = '';
		dump( 'mozphone stopped !\n' );
		return true;
	},
	started: function () { 
		mozphone_loaded = mozphone_loading;
		return mozphone_loaded;
	},
	status: function () { return mozphone_loaded; },
	write: function () { dump('Mozphone: ' + mozphone_loaded + '\n'); return true; },

/*	
	change: function (x) { this._s=x; return true; },

	_b: false,
	get b() { return this._b; },
	set b(x) { return this._b=x; },
	toggle: function (x) { return this._b = !this._b; },
*/
	QueryInterface: function (iid) {
		if (!iid.equals(Components.interfaces.nsIMozphone) &&
				!iid.equals(Components.interfaces.nsISupports)) {
			throw Components.results.NS_ERROR_NO_INTERFACE;
		}
		return this;
	}
}

// Module definition
var MozphoneComponentModule = {

	registerSelf: function (compMgr, fileSpec, location, type) {
		compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
		var catman = Components.classes["@mozilla.org/categorymanager;1"]
			.getService(nsICategoryManager);

		dump("mp_comp: *** Registering -phone handler.\n");
		compMgr.registerFactoryLocation(CLINE_SERVICE_CID,
			"Mozphone CommandLine Service",
			CLINE_SERVICE_CONTRACTID, 
			fileSpec, location, type);
		catman.addCategoryEntry("command-line-argument-handlers",
			"mozphone command line handler",
			CLINE_SERVICE_CONTRACTID, true, true);
		catman.addCategoryEntry("command-line-handler",
			"m-phone",
			CLINE_SERVICE_CONTRACTID, true, true);
 
		dump("mp_comp: *** Registering content listener.\n");
		compMgr.registerFactoryLocation(TELCONTENT_LISTENER_CID,
			"TEL content listener",
			TELCONTENT_LISTENER_CONTRACTID, 
			fileSpec, location, type);
		catman.addCategoryEntry("external-uricontentlisteners",
			TEL_MIMETYPE,
			TELCONTENT_LISTENER_CONTRACTID, true, true);

		dump("mp_comp: *** Registering tel protocol handler.\n");
		compMgr.registerFactoryLocation(TELPROT_HANDLER_CID,
			"TEL protocol handler",
			TELPROT_HANDLER_CONTRACTID, 
			fileSpec, location, type);

		dump("mp_comp: *** Registering Mozphone component ***\n");
		compMgr.registerFactoryLocation(COMPONENT_CID,
			"Mozphone component",
			COMPONENT_CONTRACTID, 
			fileSpec, location, type);

	},

	unregisterSelf: function (compMgr, fileSpec, location) {
		dump('mp_comp: unregisterSelf\n');
	},

	getClassObject: function (compMgr, cid, iid) {
		dump('mp_comp: getClassObject: ' + cid + ', iid=' + iid + '\n');
/*		// Load Mozphone preferences
      var url = "@mozilla.org/moz/jssubscript-loader;1";
		var srv = "mozIJSSubScriptLoader";
		Components.classes[url].getService(Components.interfaces[srv]).loadSubScript('chrome://mozphone/content/jslib.js');
		Components.classes[url].getService(Components.interfaces[srv]).loadSubScript('chrome://mozphone/content/preferences.js');
		var prefs = new Preferences();
		prefs.load();
		if( prefs.auto_start == 'true' && mozphone_loaded == '' ) {
			dump('mp_comp: auto_start=true, starting MozPhone !\n');
			mozphone_loaded = 'mozphone_loading';
			if (prefs.gui =='sidebar')
					browserWindow.toggleSidebar('mp_sidebar');
			else
			openWindow('chrome://mozphone/content/mozphone.xul', null);
		}
*/
		var ret;
		if (cid.equals(CLINE_SERVICE_CID))
			ret = CLineFactory;
		else if (cid.equals(TELCONTENT_LISTENER_CID))
			ret = TELContentListenerFactory;
		else if (cid.equals(TELPROT_HANDLER_CID))
			ret = TELProtocolHandlerFactory;
		else if (cid.equals(COMPONENT_CID)) {
			if (iid.equals(Components.interfaces.nsIFactory)) 
				ret = this.mpFactory;
			else {
				throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
				return;
			}
		} else {
			throw Components.results.NS_ERROR_NO_INTERFACE;
			return;
		}
		return ret;
	},

	canUnload: function (compMgr) {
		dump('mp_comp: canUnload\n');
		return true;
	},

	mpFactory: {
		createInstance: function (outer, iid) {
			dump('mp_comp: mpFactory: createInstance outer='+outer+', CID='+iid+'\n');
			if (outer!=null)
				throw Components.results.NS_ERROR_NO_AGGREGATION;
			return (new MozphoneComponent()).QueryInterface(iid);
		}
	}

}; // End Module

/* entrypoint */
function NSGetModule(compMgr, fileSpec) {
    return MozphoneComponentModule;
}
