
//-------------- Call Object
// Manages call state + UI

Call.MAXCALLS = 4;
Call.selected = -1;
Call.calls    = 0; /* Active calls */

// Access localized strings: stringbundle must be defined in document
var locales;

function Call( n, loc ) { // Constructor
	locales = document.getElementById( loc );
	// Generate UI
	var pu = document.createElement( 'popup' );
	pu.setAttribute( 'id', 'call_popup_' + n);

	var mi = document.createElement( 'menuitem' );
	mi.setAttribute( 'label', locales.getString('hangup.label') );
	mi.setAttribute( 'oncommand', 'calls[' + n + '].hangup()' );
	pu.appendChild( mi );

	pu.appendChild( create_menuitem_checkbox(
		'mi_mute_' + n,
		locales.getString('mute.label'),
		true,
		'calls[' + n + '].mute()'));

	pu.appendChild( create_menuitem_checkbox(
		'mi_hold_' + n,
		locales.getString('hold.label'),
		true,
		'calls[' + n + '].hold()'));

	mi = document.createElement( 'menuitem' );
	mi.setAttribute( 'label', locales.getString('transfer.label') + '...' );
	mi.setAttribute( 'oncommand', 'calls[' + n + '].transfer()' );
	pu.appendChild( mi );

	mi = document.createElement( 'menuitem' );
	mi.setAttribute( 'label', locales.getString('text.label') + '...' );
	mi.setAttribute( 'oncommand', 'calls[' + n + '].text()' );
	pu.appendChild( mi );

	var gb = document.createElement( 'groupbox' );
	gb.setAttribute( 'style', '-moz-border-radius: 20px; background: #aaaaaa' );
	gb.setAttribute( 'id', 'call_group_' + n );
	gb.setAttribute( 'collapsed', false ); 
	gb.setAttribute( 'onclick', 'calls[' + n + '].select()' );
	gb.setAttribute( 'context', 'call_popup_' + n );
	this.ui = gb;
	gb.appendChild( pu );
	var vb = document.createElement( 'vbox' );
	gb.appendChild( vb );
	var cc = create_description( 'cap_call_'+n, '- - - - - -', 'LCD_on', '' );
	if( prefs.hide_cid == 'true' )
		cc.setAttribute('collapsed','true')
	vb.appendChild(cc);
	vb.appendChild( create_description( 'lduration'+n, '-:--', 'LCD_on', '' ));

	/* wasim menu: X close frame with call appearance, H Hangup, M Mute, 
		O hOld, T Transfer, E tExt */
	vb = document.createElement( 'hbox' );
	vb.setAttribute( 'pack', 'center' );
	gb.appendChild( vb );
	vb.appendChild( create_description( 
		'b_close_'+n, 'X', 'LCD_on', 
		'calls[' + n + '].hide()', '' )); // locales.getString('hide.label') ));
	vb.appendChild( create_description( 
		'b_hangup_'+n, 'H', 'LCD_on', 
		'calls[' + n + '].hangup()', locales.getString('hangup.label') ));
	vb.appendChild( create_description( 
		'b_mute_'+n, 'M', 'LCD_on',
		'calls[' + n + '].bmute()', locales.getString('mute.label') ));
	vb.appendChild( create_description( 
		'b_hold_'+n, 'O', 'LCD_on',
		'calls[' + n + '].bhold()', locales.getString('hold.label') ));
	vb.appendChild( create_description( 
		'b_transfer_'+n, 'T', 'LCD_on',
		'calls[' + n + '].transfer()', locales.getString('transfer.label') + '...' ));
	vb.appendChild( create_description( 
		'b_text_'+n, 'E', 'LCD_on', 
		'calls[' + n + '].text()', locales.getString('text.label') + '...' ));

	// Properties
	this.callNo     = n;
	this.ringingout = false;
	this.active     = false;
	this.begin      = new Date();
	this.accept_dlg = false;
	this.timer	    = 0;
	this.remote	    = '';
	this.channel    = '';
	this.msg_win    = false;
	this.hold_on_activated = false;

	// Public methods
	this.activated     = activated;
	this.select        = select;
	this.text          = text;
	this.selected      = selected;
	this.hangup        = hangup;
	this.mute          = mute;
	this.hold          = hold;
	this.bmute         = bmute;
	this.bhold         = bhold;
	this.disconnected  = disconnected;
	this.ringing       = ringing;
	this.dur_timer     = dur_timer;
	this.accept_dlg_cb = accept_dlg_cb;
	this.ringout       = ringout;
	this.show          = show;
	this.hide          = hide;
	this.msg_receive   = msg_receive;
	this.transfer      = transfer;
	this.display_image = display_image;

}

function select() { // Called when user click on call area
	if( ! this.active ) return;
	if( Call.selected == this.callNo ) return; // Already selected
	proxiax_send( 'select,' + this.callNo );
}

function selected() { // Called on IAXclient select event
	this.show();
//	if( ! this.active ) return;
	if( Call.selected == this.callNo ) return; // Already selected
	Call.selected = this.callNo;
	for( var i=0; i<Call.MAXCALLS; i++ ) {
		if( i == this.callNo ) continue;
		if( document.getElementById('cap_call_' + i ) ) {
			var x = document.getElementById('cap_call_' + i );
			switch( x.getAttribute('class') ) {
				case 'LCD_mute':
					proxiax_send( 'unmute,' + i );
					// Fall through, no break
				case 'LCD_on':
					document.getElementById('cap_call_' + i ).
						setAttribute('class', 'LCD_hold');
					document.getElementById('lduration' + i ).
						setAttribute('class', 'LCD_hold');
					break;
			}
		}
	}
	var n = this.callNo;
	document.getElementById( 'call_group_' + n ).
		setAttribute( 'style', '-moz-border-radius: 20px; background: #cccccc' );
	document.getElementById('cap_call_'   + n ).setAttribute('class', 'LCD_on');
	document.getElementById('lduration'   + n ).setAttribute('class', 'LCD_on');
	document.getElementById('b_hold_'     + n ).setAttribute('class', 'LCD_on');
	document.getElementById('b_mute_'     + n ).setAttribute('class', 'LCD_on');
	document.getElementById('b_text_'     + n ).setAttribute('class', 'LCD_on');
	document.getElementById('b_transfer_' + n ).setAttribute('class', 'LCD_on');
	document.getElementById('b_hangup_'   + n ).setAttribute('class', 'LCD_on');
	document.getElementById('b_close_'    + n ).setAttribute('class', 'LCD_on');
}

function hangup() { // Called when user click on Hangup button
	if( ! ( this.active || this.ringingout ) ) return;
	if( prefs.callconf=='true' && 
			!confirm( locales.getString('confirm_hangup' ) ) )
		return;
	proxiax_send( 'hangup,' + this.callNo );
}

function msg_receive( msg ) { 
	// Called from mozphone.js on iaxc user text event
	// or when user click on Text button
	if( ! this.active ) return;
	if( this.msg_win && this.msg_win.receive ) {
		this.msg_win.receive(msg);
	} else {
		this.msg_win = 
			window.openDialog( 'chrome://mozphone/content/text.xul',
				'text_' + this.callNo, 'chrome,resizable,scrollbars', 
				this.callNo, this.remote, msg );		
	}
}

function text() { // Called when user click on Text button
	if( ! this.active ) return;
	this.msg_win = 
		window.openDialog('chrome://mozphone/content/text.xul',
			'text_' + this.callNo, 'chrome,resizable,scrollbars', 
			this.callNo, this.remote, '' );
}

function transfer() { // Called when user click on Text button
	if( ! this.active ) return;
	var s = prompt( locales.getString('transfer.prompt') );
	if( s )
		proxiax_send( 'transfer,' + this.callNo + ',' + s );
}

function bmute() { // Called when user click on Mute button
	if( ! this.active ) return;
	var n = this.callNo;
	var m = document.getElementById('mi_mute_'+ n );
	if( m.getAttribute('checked') == 'true' ) { 
		m.setAttribute('checked', false);
		proxiax_send( 'unmute,' + this.callNo );
		document.getElementById('b_mute_' + n ).setAttribute('class', 'LCD_on');
		if( document.getElementById('mi_hold_'+ n ).getAttribute('checked') != 'true') {
			document.getElementById('cap_call_' + n ).setAttribute('class', 'LCD_on');
			document.getElementById('lduration' + n ).setAttribute('class', 'LCD_on');
		}
	} else {
		m.setAttribute('checked', true);
		proxiax_send( 'mute,' + this.callNo );
		document.getElementById('b_mute_' + n ).setAttribute('class', 'LCD_mute');
			document.getElementById('cap_call_' + n ).setAttribute('class', 'LCD_mute');
			document.getElementById('lduration' + n ).setAttribute('class', 'LCD_mute');
	}
}

function mute() { // Called when user selects Mute menu item
	if( ! this.active ) return;
	var n = this.callNo;
	var m = document.getElementById('mi_mute_'+ n );
	// Test inverted ! (check status already changed when here?)
	if( ! m.getAttribute('checked') ) { 
		proxiax_send( 'unmute,' + this.callNo );
		document.getElementById('b_mute_' + n ).setAttribute('class', 'LCD_on');
		if( document.getElementById('mi_hold_'+ n ).getAttribute('checked') != 'true') {
			document.getElementById('cap_call_' + n ).setAttribute('class', 'LCD_on');
			document.getElementById('lduration' + n ).setAttribute('class', 'LCD_on');
		}
	} else {
		proxiax_send( 'mute,' + this.callNo );
		document.getElementById('b_mute_' + n ).setAttribute('class', 'LCD_mute');
		document.getElementById('cap_call_' + n ).setAttribute('class', 'LCD_mute');
		document.getElementById('lduration' + n ).setAttribute('class', 'LCD_mute');
	}
}

function bhold() { // Called when user click on Hold button
	if( ! this.active ) return;
	n = this.callNo;
	var m = document.getElementById('mi_hold_'+ n );
	if( m.getAttribute('checked') == 'true' ) { 
		m.setAttribute('checked', false);
		proxiax_send( 'unhold,' + this.callNo );
		document.getElementById('b_hold_' + n ).setAttribute('class', 'LCD_on');
		if( document.getElementById('mi_mute_'+ n ).getAttribute('checked') != 'true') {
			document.getElementById('cap_call_' + n ).setAttribute('class', 'LCD_on');
			document.getElementById('lduration' + n ).setAttribute('class', 'LCD_on');
		}
	} else {
		m.setAttribute('checked', true);
		proxiax_send( 'hold,' + this.callNo );
		document.getElementById('b_hold_' + n ).setAttribute('class', 'LCD_mute');
		document.getElementById('cap_call_' + n ).setAttribute('class', 'LCD_mute');
		document.getElementById('lduration' + n ).setAttribute('class', 'LCD_mute');
	}
}

function hold() { // Called when user selects Hold menu item
	if( ! this.active ) return;
	var n = this.callNo;
	var m = document.getElementById('mi_hold_'+ n );
	// Test inverted ! (check status already changed when here?)
	if( ! m.getAttribute('checked') ) { 
		proxiax_send( 'unhold,' + this.callNo );
		document.getElementById('b_hold_' + n ).setAttribute('class', 'LCD_on');
		if( document.getElementById('mi_mute_'+ n ).getAttribute('checked') != 'true') {
			document.getElementById('cap_call_' + n ).setAttribute('class', 'LCD_on');
			document.getElementById('lduration' + n ).setAttribute('class', 'LCD_on');
		}
	} else {
		proxiax_send( 'hold,' + this.callNo );
		document.getElementById('b_hold_' + n ).setAttribute('class', 'LCD_mute');
		document.getElementById('cap_call_' + n ).setAttribute('class', 'LCD_mute');
		document.getElementById('lduration' + n ).setAttribute('class', 'LCD_mute');
	}
}

function disconnected() { // Called on IAXclient not_active event
	if( this.accept_dlg ) this.accept_dlg.close();
	//if( !( this.active  || this.ringingout ) ) return;
	this.active     = false;
	this.ringingout = false;
	var n = this.callNo;
	var d = document.getElementById('cap_call_'+ n);
	if( d ) {
		d.setAttribute('class', 'LCD_off');
		d = document.getElementById('lduration' + n);
		d.setAttribute('class', 'LCD_off');
		document.getElementById( 'call_group_' + n ).setAttribute( 'style', '-moz-border-radius: 20px; border: 0.2em; background: #aaaaaa' );
		document.getElementById('mi_mute_'+ n ).setAttribute('checked', false);
		document.getElementById('mi_hold_'+ n ).setAttribute('checked', false);
		document.getElementById('b_hold_' + n ).setAttribute('class', 'LCD_off');
		document.getElementById('b_mute_' + n ).setAttribute('class', 'LCD_off');
		document.getElementById('b_text_' + n ).setAttribute('class', 'LCD_off');
		document.getElementById('b_transfer_' + n ).setAttribute('class', 'LCD_off');
		document.getElementById('b_hangup_' + n ).setAttribute('class', 'LCD_off');
		document.getElementById('b_close_' + n ).setAttribute('class', 'LCD_off');
		if( prefs.autohide=='true' ) {
			// Need to use closure for timeout on objects
			var call = this;
			setTimeout( function() { call.hide(); }, 5000);
		}
	}
	Call.selected   = -1;
	if( Call.calls > 0 ) Call.calls--;
}

function ringing( remote ) {
	if( Call.calls == Call.MAXCALLS ) {
		proxiax_send( 'reject,' + this.callNo );
		return;
	}
	// Add incoming call to history
	var r = remote.split(':');
	append_call_list( '<', r[0] );

	// Auto answer ?
	if( prefs.auto_answer=='true' ) {
		proxiax_send( 'answer,' + this.callNo );
		return;
	}

	// Show dialog
	var param = new Object(); // params for dialog
	param.remote = remote;
	this.remote  = r[0];
	this.channel = r[1];
	param.line   = this.callNo;
	param.call   = this;
	// Don't use modal in win_prefs as it blocks other MozPhone "threads", 
	// which results in calling party hangup not detected
	var win_prefs = 'chrome,dialog=no,titlebar=no'; 
	if (! /Mac/g.test(window.navigator.platform))
		win_prefs += ',popup';
	// Dialog is approximately 400x100
	var sY = (window.screen.height-100)/2;
	var sX = (window.screen.width-400)/2;
	win_prefs +=	',screenX=' + sX + ',screenY=' + sY;
	var chrome = 'chrome://mozphone/content/accept_dlg.xul';
	this.accept_dlg = window.openDialog(chrome, "accept_dlg", win_prefs, param);
}

function accept_dlg_cb( ret ) { // Called when accept_dlg is closed
	if( ret > 0 ) {
		proxiax_send( 'answer,' + this.callNo );
		if( ret == 2 )
			this.hold_on_activated = true;
	} else if( ret == 0 ) {
		proxiax_send( 'reject,' + this.callNo );
	}
}

function activated( remote ) {
	this.show();
	this.ringingout = false;
	if( this.accept_dlg ) this.accept_dlg.close();
	if( ! this.active ) {
		this.begin = new Date();
		var d = document.getElementById('cap_call_' + this.callNo);
		d.value = remote;
		d.setAttribute('class', 'LCD_on');
		d = document.getElementById('lduration' + this.callNo);
		d.setAttribute('class', 'LCD_on');
		this.active = true;
		this.remote = remote;
		Call.calls++;
	}
	if( this.hold_on_activated ) {
		document.getElementById( 'call_group_' + this.callNo ).
		setAttribute( 'style', '-moz-border-radius: 20px; background: #cccccc' );
		this.hold();
		this.hold_on_activated = false;
	} else {
		this.selected();
	}
}

// Called from proxyax in mozphone.js for efficiency reasons
// (just one global timer instead of MAXCALLS timers).
function dur_timer() {
	if( !this.active ) 
		return;
	var now = new Date();
	var dur = (now-this.begin) / 1000;
	var s = Math.floor( dur % 60 );
	var m = Math.floor( dur / 60 );
	if( 10 > s ) 
		s = '0' + s;
	document.getElementById('lduration' + this.callNo).value = 
		m.toString() + ':' + s;
}

function display_image( file ) {
	window.open(file, "image", 'resizable,width=100,height=100');
}

function ringout() { // Show call appearance while calling
	this.ringingout = true;
	this.selected();
	document.getElementById('cap_call_' + this.callNo).value = '- - - - -';
	document.getElementById('lduration' + this.callNo).value = '-:--';
}

function show() { // Show call appearance
	var el = document.getElementById( 'call_group_' + this.callNo );
	if( ! el ) {
		// Insert the call UI main_vbox is the parent element
		document.getElementById('main_vbox')
			.insertBefore( this.ui, document.getElementById('dialgroup') );
	} else {
		collapse( false, 'call_group_' + this.callNo);
	}
}

function hide() {
	if( ! this.active )
		collapse( true, 'call_group_' + this.callNo );
}

//-------------------------------------------------------------------
