/*
	mozphone.js
	Author: Jean-Denis Girard <jd.girard (at) sysnux.pf>
	License: Mozilla Public License or GPL

	mozphone.onLoad:
		-> mp_init: define gui objects
			-> ast_init: start network_client, load prefs, setup call objects, socket comms
*/

try {	// enablePrivilege is required if not running chrome'd
		// (other tweaks might apply, check out public.mozdev.jslib)
				netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
				include (jslib_socket);
				include (jslib_routines);
				include (jslib_file);
				include (jslib_fileutils);
				include (jslib_dir);
				include (jslib_window);
				include (jslib_dirutils);
} catch( e ) { 
    const msg = "MozIAX requires jsLib to properly operate\n"
              + "Would you like to install jsLib now? (browser restart required)";

    if (confirm(msg)) {
      const url = "http://www.mozdevgroup.com/dropbox/jslib/signed/"
                + "jslib_current_global_signed.xpi";
      try {
        _content.location = url;
      } catch (e) { open(url, "_blank", "width=1,height=1"); }
    }

    throw "Stopping any further code execution ...";
}

// Localized strings bundle, accessible via strings.getString(`foo') or 
// strings.getFormattedString("fileInfo", [fileName, fileSize])
var strings = document.getElementById( "mp_strings" );

// Array of call objects; index is line number
var calls = new Array();

// proxiax to communicate with IAX network_client
var proxiax = new Object(); 

// GUI
var gui = new Object();

// Preferences object
var prefs = new Preferences();

// MozPhone Component
var mpc;

function mp_debug( s ) {
	if(  gui.debug && gui.debug.log ) 
		gui.debug.log( s );
	else
		dump('mp_debug:'+s+'\n');
}

function new_prefs() {
	// Apply changes now
	child_windows();
	if( proxiax.sock ) {
		filters();
		codecs();
		devices();
	}
	return true;
}

function child_windows() {
	debug_window();
	gmanage_window();
	queues_window();
	fop_window();
	if( prefs.dialpad == 'true' ) {
		collapse(true, 'dialpad');
		document.getElementById( 'menu.dialpad' ).setAttribute( 'checked', 'false' );
	}
	if( prefs.levels == 'true' ) {
		collapse(true, 'levels');
		document.getElementById( 'menu.levels' ).setAttribute( 'checked', 'false' );
	}
	if( prefs.hide_cid == 'true' ) {
		collapse(true, 'call_adr');
		collapse(true, 'bcall');
	}
}

function debug_window() {
	// debug window
	if( prefs.debug == 'true') {
		if( ! gui.debug ) { // Create window
			gui.debug = window.open('chrome://mozphone/content/debug.xul', 
				'debug_main', 'chrome,resizable,scrollbars');
		}
	} else {
		if( gui.debug ) { // Delete window
			gui.debug.close();
			gui.debug = 0;
		}
	}
}

function gmanage_window() {
	// semi-graphical manager window
	if( prefs.gmanage == 'true' ) {
		if( ! gui.man_logged_in ) {
			alert( strings.getString('man_not_logged_in') );
			if( gui.gmanage ) { // Delete window
				gui.gmanage.close();
				gui.gmanage = 0;
			}
		} else if( ! gui.gmanage ) { // Create window
			gui.gmanage = window.open('chrome://mozphone/content/gmanage.xul', 
				'gmanage_main', 'chrome,resizable,scrollbars');
		}
	} else {
		if( gui.gmanage ) { // Delete window
			gui.gmanage.close();
			gui.gmanage = 0;
		}
	}
}

function queues_window() {
	// Queues window
	if( prefs.queues == 'true' ) {
		if( ! gui.man_logged_in ) {
			alert( strings.getString('man_not_logged_in') );
			if( gui.queues ) { // Delete window
				gui.queues.close();
				gui.queues = 0;
			}
		} else if( ! gui.queues ) { // Create window
			gui.queues = window.open('chrome://mozphone/content/queues.xul', 
				'queues_main', 'chrome,resizable,scrollbars');
		}
	} else {
		if( gui.queues ) { // Delete window
			gui.queues.close();
			gui.queues = 0;
		}
	}
}

function fop_window() {
	// Flash Operator Panel
	if( prefs.fop != '' ) {
		if( ! gui.fop ) { // Create window
			gui.fop = window.open( prefs.fop, 'fop_main', 'resizable');
		}
	} else {
		if( gui.fop ) { // Delete window
			gui.fop.close();
			gui.fop = 0;
		}
	}
}

function mp_init() { // MozPhone initialization (called by onLoad event)

	try {
		mpc = Components.classes["@mozilla.org/mozphone_component;1"].
			createInstance(Components.interfaces.nsIMozphone);
		dump( 'Mozphone Component loaded!\n' );
	} catch(e) {
		dump( 'Mozphone Component ERROR: ' + e + '\n' );
	}
	if (mpc.status()!='') {// Already running !
		dump( 'mp_init: Already running, exiting\n' );
		mp_close();
		return;
	}
	mpc.started();

	// Splash screen !
	mp_splash( 'chrome://mozphone/content/splash.xul', 'MozPhone loading...' );

	// Populate gui object
	gui.call_adr   = document.getElementById( "call_adr" );
	gui.bcall      = document.getElementById( "bcall" );
	gui.in_meter   = document.getElementById( "in_meter" );
	gui.out_meter  = document.getElementById( "out_meter" );
	gui.registration = document.getElementById( "registration" );
//	gui.call_adr.focus();
	gui.GUID = '4de2cc0c-a9e4-4853-871c-7cda72582e13'; // MozPhone GUID
	gui.man_logged_in = false;

	// Extended dialogs through PromptService
	gui.prompts = jslibGetService("@mozilla.org/embedcomp/prompt-service;1",
                                 "nsIPromptService");
	// "Quit" no longer exported in Firefox 1.0 ?
	if( typeof(window.quit)=='function' ) {
		// Append quit menuitem
		menu = document.getElementById( "menu-popup" );
		// <menuitem label="&menu.quit;" oncommand="mp_quit();quit();" />
		var item = document.createElement( 'menuitem' );
		item.setAttribute( 'label', strings.getString('quit') );
		item.setAttribute( 'oncommand', 'mp_close();quit();' );
		menu.appendChild(item);
	}

	//  Then load preferences
	prefs.load();
	debug_window(); // Open debug window ASAP
	var list = gui.call_adr;
	for( var i=0; i < prefs.history.length; i++ ) {
		if( (!list.childNodes[0]) ||
				list.childNodes[0].childNodes.length < prefs.history_max ) {
			var x = prefs.history[i].split(' ');
			if( x.length < 3 ) {
				x[2] = '2000/01/01';
				x[3] = '00:00:00';
			}
			var item = list.appendItem( x[0] + ' ' + x[1], x[1] + ' ' + x[2] + ' ' + x[3] );
			item.setAttribute( 'tooltiptext', x[2] + ' ' + x[3] );
		}
	}

	// Then instantiate call objects
	if( prefs.concurrency ) {
		if( prefs.concurrency > Call.MAXCALLS ) 
			prefs.concurrency = Call.MAXCALLS;
		else 
			Call.MAXCALLS = prefs.concurrency;
	} else 
			prefs.concurrency = Call.MAXCALLS;
	for( var i=0; i<Call.MAXCALLS+1; i++ )
		calls[i] = new Call(i, 'call_strings');

	// Then start asterisk communications through network_client
	ast_init();
}

function mp_splash( url ) {
	var win = new CommonWindow( url, 350, 300, msg );
	win.position = JS_MIDDLE_CENTER;
	win.openSplash();
}

function debug_close() { // Called on unload event from debug window
	gui.debug = 0;
}

function gmanage_close() { // Called on unload event from gmanager window
	gui.gmanage = 0;
}

function queues_close() { // Called on unload event from queues window
	gui.queues = 0;
}

function fop_close() { // Called on unload event from fop window
	gui.fop = 0;
}

// Show preferences dialog
function prefs_dlg() {
	var ret = new Object(); // Dialog returning value
	ret.res = 0;
	window.openDialog('chrome://mozphone/content/preferences.xul',	'prefs_main', 'chrome,resizable,scrollbars,dialog=yes,close,modal=yes', prefs, ret);
	if(ret.res) {
		if(ret.res==2 && proxiax.sock)
			register();
		new_prefs();
	}
}

function ast_init() {
	// Search for proxiax host / port in environment
	proxiax.host = '127.0.0.1'; // Default values
	proxiax.port = 9998;
	var env = Components.classes["@mozilla.org/process/environment;1"].
		getService(Components.interfaces.nsIEnvironment);
	if( env.exists('PROXIAX_HOST') )	 {
		prefs.image_dir = ''; // No image when running remote network_client !
		proxiax.host = env.get('PROXIAX_HOST');
		mp_debug( 'Found environment variable PROXIAX_HOST: ' +  proxiax.host + '<br/>\n' );
		if( env.exists('PROXIAX_PORT') )
			proxiax.port = env.get('PROXIAX_PORT');
	}

	if( proxiax.host == '127.0.0.1' ) {
		// Look for network_client executable and spawn it 
		// only if on same host !
		var nc_ok = 0;
		var os = navigator.platform;
		var name = 'network_client';
		var du = new DirUtils();
		var fu = new FileUtils();
		var sdir = new Dir( du.getCurProcDir() );
		var cdir = new Dir( du.getComponentsDir() );
		var udir = new Dir( du.getPrefsDir() );
		sdir = sdir.path;
		cdir = cdir.path;
		udir = udir.path;
		var network_client;

		if( /win/i.test(os) ) { // Running on Windows
			// Hack to hide DOS console by launching network_client.exe 
			// through CreateProcess in nowin.exe
			// Is there a better way ?
			var no_win;
			name += '.exe'; 

			// Try in user dir first
			var dir = udir + '\\extensions\\{' + gui.GUID + '}\\components\\';
			network_client = dir + name;
			if( fu.exists(network_client) ) {
				nc_ok++;
				no_win = dir + 'nowin.exe';
				if( fu.exists(no_win) )
					fu.run( no_win, [ network_client ] );
				else
					fu.run( network_client );
			}
			if (!nc_ok) { // Then try in global dir
				dir = sdir + '\\extensions\\{' + gui.GUID + '}\\components\\';
				network_client = dir + name;
				if( fu.exists(network_client) ) {
					nc_ok++;
					no_win = dir + 'nowin.exe';
					if( fu.exists(no_win) )
						fu.run( no_win, [ network_client ] );
					else
						fu.run( network_client );
				}
			}
			if (!nc_ok) { // Last chance, try in components dir
				network_client = cdir + '\\' + name;
				if( fu.exists(network_client) ) {
					nc_ok++;
					no_win = cdir + '\\nowin.exe';
					if( fu.exists(no_win) ) {
						fu.run( no_win, [ network_client ] );
					} else {
						fu.run( network_client );
					}
				}
		}

		// } else if( /mac/i.test(os) )  XXX
		// XXX What about Running on Mac ?

		} else { // Running on Linux
			// Try in user dir first
			var dir = udir + '/extensions/{' + gui.GUID + '}/components/';
			network_client = dir + name;
			if( fu.exists(network_client) ) {
				nc_ok++;
				fu.run( network_client );
			}
			if (!nc_ok) { // Then try in global dir
				dir = sdir + '/extensions/{' + gui.GUID + '}/components/';
				network_client = dir + name;
				if( fu.exists(network_client) ) {
					nc_ok++;
					fu.run( network_client );
				}
			}
			if (!nc_ok) { // Last chance, try in components dir
				network_client = cdir + '/' + name;
				if( fu.exists(network_client) ) {
					nc_ok++;
						fu.run( network_client );
				}
			}

		}

		if (!nc_ok) {
			alert( strings.getString('nc_error') );
			mp_close();
		}
	}

	// Now try to connect to network_client
	proxiax.retry = 0;
	proxiax.data = '';
	proxiax.sock = new Socket();
	setTimeout( "open_socket()", 100 ); // Wait for network_client start
}

function open_socket( )  {
	if( proxiax.sock.isConnected ) 
		return;
	proxiax.sock.open( proxiax.host, proxiax.port, true ); // Open returns nothing !
	mp_debug( 'proxiax socket ' +  proxiax.host + ":" + proxiax.port + ' openned...<br/>' );
	setTimeout( "proxiax_connect()", 100 );
}

function proxiax_connect() {
	// Connect to network_client
	if( ! proxiax.sock.isAlive() ) {
		proxiax.retry++;
		mp_debug( '<span style="color:red">Socket not alive, ' 
			+ proxiax.retry + ' !</span><br/>');

		if( proxiax.retry == 50 ) { // No connection, quit now !
			alert( strings.getString('proxiax_conn') );
			mp_close();
		}

		if( (proxiax.retry % 10) == 0) { // Reopen socket and try again
			proxiax.sock.close();
			mp_debug( 'Socket was closed<br/>' );
			proxiax.sock = new Socket();
			setTimeout( "open_socket()", 100 );
			return;
		}
		setTimeout( "proxiax_connect()", 50 );
		return;
	} 
	mp_debug( '<span style="color:green">Socket alive on retry ' 
			+ proxiax.retry + ' !</span><br/>');
	proxiax.retry = 0;
	proxiax_synchro();
}

function proxiax_synchro() {
	// Synchronize with network_client
	var data = receive_socket( proxiax.sock );
	if( data ) {
		proxiax.data += data;
	}
	var iaxcver = proxiax.data.match( /Hello\s(.*)/ ); 
	if( ! iaxcver ) {
		mp_debug( 'proxiax waiting for synchro, retry ' + proxiax.retry + '... "' 
				+ proxiax.data + '"<br/>' );
		if( proxiax.retry >= 20 ) { // No synchro, quit now !
			alert( strings.getString('proxiax_sync') );
			mp_close();
		}
		proxiax.retry++;
		setTimeout( "proxiax_synchro()", 100 );
		return;
	}
	proxiax.iaxcver = iaxcver[1];
	mp_debug( 'proxiax synchro OK, iaxclient_version ' + proxiax.iaxcver + '<br/>\n' );

	// ProxIAX should be OK now, start receiving "thread"
	proxiax.data = '';
	proxiax_read();

	proxiax_send( 'image_dir,' + prefs.image_dir );
	proxiax_send( 'iaxc_initialize,' + Call.MAXCALLS );
	register()
	codecs();

	// Connect to asterisk manager (same host)
	if( prefs.man_username!='' && prefs.man_secret!='' )
		proxiax_send( 'man_login,' + prefs.man_username + ',' + prefs.man_secret );

	// Get / Set list of audio devices: list of devices will be obtained asynchronously
	// so we can't rely on it here, just use blindly the prefs
	proxiax_send( 'get_devices' );
	devices();

	// Apply filters
	filters();

	// Display other windows (delay needed for manager login)
	setTimeout( 'child_windows()', 1000 );

	// Did we receive arguments ?
	if( window.arguments && window.arguments[0] ) {
		gui.call_adr.value = window.arguments[0].url;
      call();
	}
}

function proxiax_read() {
	var data = receive_socket( proxiax.sock );
	if( data ) {
		data = proxiax.data + data;
		var index = data.lastIndexOf( '\r\n' );
		if( index != data.length-2 ) { // Message is not complete
			proxiax.data = data.slice( index+2 );
		}
		var strings = new Array;
		strings = data.split( '\r\n' );
		for( var i=0; i<strings.length-1 ; i++ ) {
			proxiax_decode( strings[i] );
		}
	}

	// Update call(s) duration(s)
	for( var i=0; i<Call.MAXCALLS; i++ ) {
		calls[i].dur_timer();
	}

	// Restart timer
	proxiax.timer = setTimeout( "proxiax_read()", 100 ); // XXX
}

function open_browser( uri, call_num ) {	
	
	// var a = /^chrome:\/\/.*\?(.*)/.exec(uri); // chrome://xxxx/yyy?p=12

	if( ! prefs.browser_path ) {
		// Native Browser Window
		var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
			.getService(Components.interfaces.nsIWindowMediator);
		var win = wm.getMostRecentWindow('navigator:browser');
		if(win) { 
			switch(prefs.browser_pref) {
				case 'browser_new_tab':
					// gBrowser is tabbrowser
					win.gBrowser.selectedTab.location = win.gBrowser.addTab(uri);
					break;
				case 'browser_replace':
					// Replace existing content
					win.content.location = uri;
					break;
				case 'browser_new_win':
				default:
					// Open new window
					open( uri );
					break;
			}
		} else {
			// No available browser, open new one
			open( uri );
		}
	} else {
		// External browser !
		var fu = new FileUtils();
		if( ! fu.exists(prefs.browser_path) ) {
			alert('ERROR: browser not found:\n' + prefs.browser_path );
		} else {
			var rv = fu.spawn( prefs.browser_path, [uri] );
			mp_debug( prefs.browser_path + ' ' + uri + ' (rv=' + rv + ')<br>' );
		}
	}
}

function proxiax_decode( s ) {
	var a = s.split( ',' );
	switch( a[0] ) {
		case 'l':
		case 'levels':
			// Levels -60..0 => 0..100
			gui.in_meter.value  = (60+parseInt(a[1]))*100/60;
			gui.out_meter.value = (60+parseInt(a[2]))*100/60;
			break;
		case 'r':
		case 'registration':
			// Result of IAXclient registration to * server
			switch( a[1] ) {
				case 'ack': // IAX event registration ack
					gui.registration.setAttribute('style','color: green');
					gui.registration.setAttribute('tooltiptext', 
							strings.getString('registration_ack'));
					proxiax.registered = true;
					break;
				case 'rej': // IAX event registration reject
				case 'failed': // IAXclient error
					gui.registration.setAttribute('style','color: red');
					alert( strings.getString('registration_error') );
					gui.registration.setAttribute('tooltiptext', 
							strings.getString('registration_error'));
					proxiax.registered = false;
					break;
				case 'tmo': // IAX event registration timeout
					gui.registration.setAttribute('style','color: yellow');
					gui.registration.setAttribute('tooltiptext', 
							strings.getString('registration_tmo'));
					proxiax.registered = false;
					break;
				case 'id': // IAXclient registration ID
					proxiax.regid = a[2];
					break;
				default:
					mp_debug( 'registration ??? ' + s + '<br>\n' );
					break;
			}
			break;
		case 'd':
		case 'device':
			// New device, parameters: number,name,capacities
			var dev = new Object();
			dev.name = a[2];
			dev.caps = a[3];
			prefs.devices[a[1]] = dev;
			break;
		case 's':
		case 'state':
			/* State event: state,callNo,cause,parameters
				cause,parameters is one of: 
					not_active
					not_accepted
					ring_out
					active,remote
					ring_in,remote
					url,url
			*/
			var call_num = parseInt( a[1] );
			var cause = a[2];
			// mp_debug( 'state: ' + cause + ' (' + call_num + ')<br>\n' );
			switch( cause ) {
				case 'ring_out':
					calls[call_num].ringout();
					break;
				case 'active':
					calls[call_num].activated( a[3] );
					break;
				case 'selected':
					calls[call_num].selected();
					break;
				case 'ring_in': // Ringing in !
					calls[call_num].ringing( a[3] );
					break;
				case 'not_active': // Disconnected !
					calls[call_num].disconnected();
					break;
				case 'url': // Display received URL in preferred browser
					uri = a[3];
					if( uri )
						open_browser( uri, call_num );
					else
						mp_debug( 'state: received url but NO URI !<br>\n' );
					break;
				case  'not_accepted': // Disconnected !
					calls[call_num].ringout(); // XXX what should be done here ?
					break;
				default: // Unknown state !
					mp_debug( 'Unknown state=' + cause +  ' ' + call_num + '<br>\n' );
					break;
			}
			break;
		case 't':
		case 'text':
/* #define IAXC_TEXT_TYPE_ERROR            3
FATAL ERROR: User Agent should probably display error, then die.
#define IAXC_TEXT_TYPE_FATALERROR       4
#define IAXC_TEXT_TYPE_IAX              5 */
			var type = a[2];
			var msg = a.slice(3).join(',');
			if( type == 3 ) {
				alert( msg );
				break;
			}
			if( type == 4 ) {
				alert( msg );
				mp_close();
				break;
			}
			if( type == 5 ) {
				var callNo = parseInt( a[1] );
				calls[callNo].msg_receive( msg );
				break;
			}
			break;
		case 'm':
		case 'manager':
			// Manager event: manager,type,data
			var type = a[1];
			var data = a[2];
			switch( type ) {
				case 'pong':
					setTimeout( function() { proxiax_send( 'man_ping' ) }, 60000 );
					break;
				case 'login':
					if( data == 'success' ) {
						gui.man_logged_in = true;
						setTimeout( function() { proxiax_send( 'man_ping' ) }, 60000 );
					} else {
						alert( strings.getString('man_login_failed') );
					}
					break;
				case 'show_message':
					if( gui.man_logged_in && data.substr(0,13) == 'Disconnected|') {
						gui.man_logged_in = false;
						mp_debug( 'show_message: ' + data + '<br>\n' );
						alert('Disconnected from manager, will try to reconnect !' );
						proxiax_send( 'man_login,' + prefs.man_username + ',' + prefs.man_secret );
					}
					break;
				default:
					man_decode( type, data );
			}
			break;
		case 'n':
		case 'number':
			// Number received from tel device
			gui.call_adr.value = a[1];
			break;
		case 'c':
		case 'call':
			// Call received from tel device
			mp_debug( 'Call :' + s + '<br>\n' );
			call();
			break;
		case 'i':
		case 'image':
			var callNo = parseInt( a[1] );
			var type = parseInt( a[2] );
			var len = parseInt( a[3] );
			if (type == (1<< 16)) type = 'jpg'
			else if (type == (1<< 17)) type = 'png'
			else {
				type = 'unknown';
				mp_debug( 'Image unknown type, ignoring<br>\n' );
				break;
			}
			var file = a[4];
			if (file == 'ERROR') {
				mp_debug( 'Image file ERROR, ignoring<br>\n' );
				break;
			}
			mp_debug( 'Displaying image ' + type + ', ' + len + ', ' + file + '<br>\n' );
			calls[callNo].display_image( 'file://' + file );
			break;
		default:
			mp_debug( 'PROXIAX !' + s + '! ???<br>\n' );
			break;
	}
}

function man_decode( type, data ) {
	if( ! gui.man_logged_in ) 
		return;
	if( gui.gmanage && gui.gmanage.decode ) {
		gui.gmanage.decode( type, data );
	}
	if( gui.queues && gui.queues.decode ) {
		gui.queues.decode( type, data );
	}
}

function proxiax_send( msg ) {
	proxiax.sock.write( msg + '\r\n' ); 
}

function receive_socket( sock ) {
	var bytes = sock.available();
	if( ! bytes ) return null;
	return sock.read( bytes );
}

// Manage IAXclient filters
function filters() {
	var x = '';
	if( prefs.AGC == 'true' ) x += 'A';
	if( prefs.EC  == 'true' ) x += 'E';
	if( prefs.DN  == 'true' ) x += 'D';
	proxiax_send( 'filters,' + x );
	proxiax_send( 'silence,' + prefs.silence );
	return true;
}

// Register as IAX peer with asterisk server
function register() {
	proxiax_send( 'iax_register,' + prefs.iax_peer + ',' + 
		prefs.iax_secret + ',' + prefs.ast_host );
	gui.registration.label = prefs.ast_host;
}

// IAX codecs
function codecs() {
	var allow = '';
	allow += (prefs.ilbc=='true')  ? 'ilbc ' :'';
	allow += (prefs.speex=='true') ? 'speex ':'';
	allow += (prefs.gsm=='true')   ? 'gsm '  :'';
	allow += (prefs.alaw=='true')  ? 'alaw ' :'';
	allow += (prefs.ulaw=='true')  ? 'ulaw ' :'';
	var cmd = 'set_formats,' + prefs.pref_codec + ',' + allow;
	proxiax_send( cmd );
}

// IAX devices
function devices() {
	var cmd = 'set_devices,' + prefs.input + ',' + prefs.output + ',' + prefs.ring;
	proxiax_send( cmd );
}

function digit( c ) {
	// If no call is selected, digit is added to call number
	// else digit is sent in selected call
	if( Call.selected == -1 )
		gui.call_adr.value += c;
	else
		proxiax_send( c );
	return true;
}


function key( event ) {
	if( event.keyCode == event.DOM_VK_RETURN ) {
		call();
		return;
	}
/*
	if( event.charCode == 0 )
		return true;
	// event.stopPropagation();
	var num = gui.call_adr.value + String.fromCharCode(event.charCode);
	var list = gui.call_adr.childNodes[0];
	for( var i=0; i<list.childNodes.length; i++ ) {
		if( list.childNodes[i].value) {
			var n = (list.childNodes[i].value).split(' ');
			if(  num == n[0].substr(0,num.length) ) {
				gui.call_adr.value = n[0];
				// gui.call_adr.XXX
				return false;
			}
		}
	}
	return true;
	for( i in event )
		mp_debug( 'event.' + i + ' = ' + event[i] + ' !<br>' );
*/
}

function append_call_list( s, num ) {
	// Timestamp
	var d = new Date();
	var yy = d.getFullYear();
	var mo = d.getMonth();
	var da = d.getDate();
	var ho = d.getHours();
	var mi = d.getMinutes();
	var se = d.getSeconds();
	var date = yy + ((mo<10) ? '/0':'/') + mo + ((da<10) ? '/0':'/') + da +
		((ho<10) ? ' 0':' ') + ho + ((mi<10) ? ':0':':') + mi + ((se<10) ? ':0':':') + se;
	// Drop down call history
	var list = gui.call_adr;

	// Insert incoming call at beginning of list
	var item = list.insertItemAt(0, s + ' ' + num, num + ' ' + date);
	item.setAttribute( 'tooltiptext', date);

	// Remove call from list if stored already
	list = list.childNodes[0];
	for( var i=1; i<list.childNodes.length; i++ ) {
		if( list.childNodes[i].value) {
			var n = (list.childNodes[i].value).split(' ');
			if(  num == n[0] )
				list.removeChild( list.childNodes[i] );
		} else
			mp_debug( 'call log ' + i + ' undefined !<br>' );
	}

	// Limit list length
	while( list.childNodes.length > prefs.history_max ) 
		list.removeChild( list.lastChild );

	prefs.history.unshift(s + ' ' + num + ' ' + date);
	for( var i=1; i<prefs.history.length; i++ ) {
		var n = prefs.history[i].split(' ');
		if(  num == n[1] )
				prefs.history.splice(i,1);
	}
	while( prefs.history.length > prefs.history_max ) 
		prefs.history.pop();
}

function call() {
	var uri = gui.call_adr.value;
mp_debug( 'New call: ' + uri );
	// XXX Supprimer ce mcanisme ?
	// uri: 123456|command?param=val
	//			\		 \command for active call
	//        \tel number to call
	var slash = uri.indexOf('|');
	var num, path;
	if( slash > -1 ) {
		num  = uri.substring( 0,slash );
		path = uri.substring( slash+1 );
	} else {
		num = uri;
		path = '';
	}
mp_debug( ', num=' + num + ', path='+ path + '.<br />' );

	// Apply commands to active call 
	//	e.g. tel://123654|transfer_call?dst=987456 makes new call to 123654 and 
	// transfers active call to 987456
	if( path != '' ) {
		var a = new Array;
		a = path.split( '?' );
		switch( a[0] ) {
			case 'transfer': // Transfer, no new call
			case 'transfer/': // Firefox adds a '/'
			case 'transfer_call': // Transfer + new call
			case 'transfer_call/':
				if( Call.selected<0 )
					break;
				var p = new Array;
				p = a[1].split( '=' );
				if( p[0] == 'dst' ) {
					mp_debug( a[0] + ',' + Call.selected + ',' + a[1] + '<br />' );
					proxiax_send( 'transfer,' + Call.selected + ',' + p[1] );
				}
				if(a[0]=='transfer' || a[0]=='transfer/')
					return false;
				break;
		default:
			mp_debug( 'Unknown call command: ' + a[0] + ', params: ' + a[1] + ' for call ' + Call.selected + '<br />' );
			break;
		}
	}

	// Now really try to make new call
	if( Call.calls >= Call.MAXCALLS ) {
		alert(strings.getString('call_max'));
		return false;
	}

	// Clean up and check dialed number
	num = num.replace( /[^\w@\.:\/_\-\*#]/g, '' );
	if( num.length < 1 ) {
		alert( strings.getString('no_num') );
		gui.call_adr.focus();
		return false;
	}

	// Call confirmation
	var check = {};
	if( prefs.callconf=='true' && 
		! gui.prompts.confirmCheck( window,
						strings.getString('confirm_title'),
						strings.getFormattedString('confirm_msg', [num]),
						strings.getString('confirm_stop'),
						check ) )
		return false;
	if( check.value )
		prefs.callconf = 'false';

	append_call_list( '>', num );

	// Make dial string
	if( num.search( /@/ )==-1 ) 
		// Dial through default IAX account
		num = prefs.iax_peer + ':' + prefs.iax_secret + '@' + 
				prefs.ast_host + '/' + num;

	// Finally call...
	proxiax_send( 'call,' +  num );
	gui.call_adr.value = '';

	return true;
}

function mp_close() {
	prefs.save();
	if( gui.gmanage ) gui.gmanage.close();
	if( gui.queues  ) gui.queues.close();
	if( gui.fop     ) gui.fop.close();
	if( gui.help    ) gui.help.close();
	if( gui.debug   ) gui.debug.close();
	mp_splash( 'chrome://mozphone/content/splash_close.xul' );
	setTimeout( "mp_close2()", 500 );
	if (mpc.status()=='sidebar') {
		var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].
					getService(Components.interfaces.nsIWindowMediator);
		var browserWindow = wm.getMostRecentWindow("navigator:browser");
		browserWindow.toggleSidebar('mp_sidebar');
//	} else {
//		close();
	}
}

// MozPhone close
function mp_close2() {
	if( proxiax.sock && proxiax.sock.isAlive() ) {
		proxiax_send( 'quit' );
	}
	// XXX kill proxiax.timer
	mpc.stop();
	close();
}

function mp_about() {
	window.openDialog('chrome://mozphone/content/about.xul', 'about_main', 
			'chrome,resizable,dialog=yes,close,modal=yes' );
}

function mp_help() {
	gui.help = window.open( 'chrome://mozphone/locale/help/index.xhtml', 
			'help', 'resizable,scrollbars');
}

function collapse(state,element) {
	// Show / hide GUI element: state=true to collapse
	var el  = document.getElementById(element);
	/*var mvb = document.getElementById(main_vbox);
   mp_debug( 'main_vbox is ' + mvb + '<br />' );*/
	if( state ) {
		if( el.getAttribute('collapsed') )
			return; // already collapsed !
		self.resizeBy( 0, -el.boxObject.height );
		el.setAttribute('collapsed', true );
		prefs[element] = 'true';
		return;
	} else {
		if( ! el.getAttribute('collapsed') )
			return; // already not collapsed !
		el.removeAttribute( 'collapsed' );
		self.resizeBy( 0, el.boxObject.height );
		prefs[element] = 'false';
	}
}

function create_el( epyt, id ) {
	var el = document.createElement( epyt );
	el.setAttribute( 'id', id );
	return el;
}

function create_description( id, value, ssalc, action, tt ) {
	var desc = create_el( 'description', id );
	if( value )
		desc.setAttribute( 'value', value );
	if( ssalc )
		desc.setAttribute( 'class', ssalc );
	if( action )
		desc.setAttribute( 'onclick', action );
	if( tt )
		desc.setAttribute( 'tooltiptext', tt );
	return desc;
}

function create_menuitem_checkbox( id, label, auto, cmd) {
	var mi = create_el( 'menuitem', id );
	mi.setAttribute( 'label', label );
	mi.setAttribute( 'type', 'checkbox' );
	mi.setAttribute( 'checked', false );
	mi.setAttribute( 'autocheck', auto );
	mi.setAttribute( 'oncommand', cmd );
	return mi;
}

function create_button( id, label, ssalc, action, tt, img ) {
	var b = create_el( 'button', id );
	if( label )
		b.setAttribute( 'label', label );
	if( ssalc )
		b.setAttribute( 'class', ssalc );
	if( action )
		b.setAttribute( 'onclick', action );
	if( tt )
		b.setAttribute( 'tooltiptext', tt );
	if( img )
		b.setAttribute( 'image', img );
	return b;
}

